module Forecast
  class Base
    attr_reader :request, :credentials

    # @see Forecast.client
    # @see Forecast.hardy_client
    # def initialize({forecast_api_id: nil, access_token: nil})
    def initialize(ops={})
      @credentials = if ops[:forecast_account_id] and ops[:access_token]

        #
        # httparty stops the party (haha) if the access_token is not a String
        #
        string_ops = {
            forecast_account_id: ops[:forecast_account_id].to_s,
            access_token: ops[:access_token],
        }

        OAuthCredentials.new(string_ops)
      else
        fail 'You must provide either :forecast_account_id and :access_token'
      end
    end

    # people - @todo
    # # All API Actions surrounding Users
    # #
    # # == Examples
    # #  harvest.users.all() # Returns all users in the system
    # #
    # #  harvest.users.find(100) # Returns the user with id = 100
    # #
    # #  user = Harvest::User.new(:first_name => 'Edgar', :last_name => 'Ruth', :email => 'edgar@ruth.com', :password => 'mypassword', :timezone => :cst, :admin => false, :telephone => '444-4444')
    # #  saved_user = harvest.users.create(user) # returns a saved version of Harvest::User
    # #
    # #  user = harvest.users.find(205)
    # #  user.email = 'edgar@ruth.com'
    # #  updated_user = harvest.users.update(user) # returns an updated version of Harvest::User
    # #
    # #  user = harvest.users.find(205)
    # #  harvest.users.delete(user) # returns 205
    # #
    # #  user = harvest.users.find(301)
    # #  deactivated_user = harvest.users.deactivate(user) # returns an updated deactivated user
    # #  activated_user = harvest.users.activate(user) # returns an updated activated user
    # #
    # #  user = harvest.users.find(401)
    # #  harvest.users.reset_password(user) # will trigger the reset password feature of harvest and shoot the user an email
    # #
    # # @see Harvest::Behavior::Crud
    # # @see Harvest::Behavior::Activatable
    # # @return [Harvest::API::Users]
    # def users
    #   @users ||= Harvest::API::Users.new(credentials)
    # end

    # All API Actions surrounding Projects
    #
    # == Examples
    #  forecast.projects.all() # Returns all projects in the system
    #
    #  harvest.projects.find(100) # Returns the project with id = 100
    #
    #  project = Harvest::Project.new(:name => 'SuprGlu' :client_id => 10)
    #  saved_project = harvest.projects.create(project) # returns a saved version of Harvest::Project
    #
    #  project = harvest.projects.find(205)
    #  project.name = 'SuprSticky'
    #  updated_project = harvest.projects.update(project) # returns an updated version of Harvest::Project
    #
    #  project = harvest.project.find(205)
    #  harvest.projects.delete(project) # returns 205
    #
    #  project = harvest.projects.find(301)
    #  deactivated_project = harvest.projects.deactivate(project) # returns an updated deactivated project
    #  activated_project = harvest.projects.activate(project) # returns an updated activated project
    #
    #  project = harvest.projects.find(401)
    #  harvest.projects.create_task(project, 'Bottling Glue') # creates and assigns a task to the project
    #
    # @see Harvest::Behavior::Crud
    # @see Harvest::Behavior::Activatable
    # @return [Harvest::API::Projects]
    def projects
      @projects ||= Forecast::API::Projects.new(credentials)
    end

    # All API Actions surrounding Assignments
    #
    # == Examples
    #  forecast.projects.all() # Returns all projects in the system
    #  forecast.projects.all({start_date: '2016-11-28', end_date: '2017-01-01', state: 'active'})
    #
    #  forecast.projects.find(100) # Returns the project with id = 100
    #
    #
    # @see Forecast::Behavior::Crud
    # @return [Forecast::API::Projects]
    def assignments
      @assignments ||= Forecast::API::Assignments.new(credentials)
    end

    # def milestones
    #   @milestones ||= Forecast::API::Milestones.new(credentials)
    # end

    def aggregates
      @aggregates ||= Forecast::API::Aggregates.new(credentials)
    end
  end
end
