require 'httparty'
require 'base64'
require 'delegate'
require 'hashie'
require 'json'
require 'time'
require 'csv'

require 'ext/array'
require 'ext/hash'
require 'ext/date'
require 'ext/time'

require 'forecast/version'
require 'forecast/credentials'
require 'forecast/errors'
require 'forecast/hardy_client'
require 'forecast/timezones'

require 'forecast/base'

%w(crud activatable).each {|a| require "forecast/behavior/#{a}"}

%w(model project assignment aggregate).each {|a| require "forecast/#{a}"}
%w(base projects assignments aggregates).each {|a| require "forecast/api/#{a}"}

module Forecast
  class << self

    # Creates a standard client that will raise all errors it encounters
    #
    # == Options
    # * Basic Authentication
    #   * +:subdomain+ - Your HarvestForecast subdomain
    #   * +:username+ - Your HarvestForecast username
    #   * +:password+ - Your HarvestForecast password
    # * OAuth
    #   * +:access_token+ - An OAuth 2.0 access token
    #
    # == Examples
    #   Forecast.client(subdomain: 'mysubdomain', username: 'myusername', password: 'mypassword')
    #   Forecast.client(access_token: 'myaccesstoken')
    #
    # @return [Forecast::Base]
    # def client(subdomain: nil, username: nil, password: nil, access_token: nil)
    def client(ops={})
      # Forecast::Base.new(subdomain: subdomain, username: username, password: password, access_token: access_token)
      Forecast::Base.new(ops)
    end

    # Creates a hardy client that will retry common HTTP errors it encounters and sleep() if it determines it is over your rate limit
    #
    # == Options
    # * Basic Authentication
    #   * +:subdomain+ - Your HarvestForecast subdomain
    #   * +:username+ - Your HarvestForecast username
    #   * +:password+ - Your HarvestForecast password
    # * OAuth
    #   * +:access_token+ - An OAuth 2.0 access token
    #
    # * +:retry+ - How many times the hardy client should retry errors. Set to +5+ by default.
    #
    # == Examples
    #   Forecast.hardy_client(subdomain: 'mysubdomain', username: 'myusername', password: 'mypassword', retry: 3)
    #
    #   Forecast.hardy_client(access_token: 'myaccesstoken', retries: 3)
    #
    # == Errors
    # The hardy client will retry the following errors
    # * Forecast::Unavailable
    # * Forecast::InformHarvest
    # * Net::HTTPError
    # * Net::HTTPFatalError
    # * Errno::ECONNRESET
    #
    # == Rate Limits
    # The hardy client will make as many requests as it can until it detects it has gone over the rate limit. Then it will +sleep()+ for the how ever long it takes for the limit to reset. You can find more information about the Rate Limiting at http://www.getforecast.com/api
    #
    # @return [Forecast::HardyClient] a Harvest::Base wrapped in a Harvest::HardyClient
    # @see Forecast::Base
    # def hardy_client(subdomain: nil, username: nil, password: nil, access_token: nil, retries: 5)
    def hardy_client(ops={}, retries=5)
      # Forecast::HardyClient.new(client(subdomain: subdomain, username: username, password: password, access_token: access_token), retries)
      Forecast::HardyClient.new(client(ops), retries)
    end
  end
end
